##########################################################################
# NSAp - Copyright (C) CEA, 2013 -2016
# Distributed under the terms of the CeCILL-B license, as published by
# the CEA-CNRS-INRIA. Refer to the LICENSE file or to
# http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
# for details.
##########################################################################

"""
Module that provides a binding over the 'dcm2nii' command. This module
also defines custom exceptions.
"""
