##########################################################################
# NSAp - Copyright (C) CEA, 2015 - 2016
# Distributed under the terms of the CeCILL-B license, as published by
# the CEA-CNRS-INRIA. Refer to the LICENSE file or to
# http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
# for details.
##########################################################################

"""
This module is proposes three main functionalitites:

* a functionality to convert DICOM files to Nifti format.
* a functionality to split DICOM files by serie names.
* a functionality to transcode the subject identifiers.
"""
